# -*- coding: utf-8 -*-

from scienco.indexes.constants import ARI_EN
from scienco.indexes.constants import ARI_RU
from scienco.indexes.constants import CLI_EN
from scienco.indexes.constants import CLI_RU
from scienco.indexes.constants import FRES_EN
from scienco.indexes.constants import FRES_RU
from scienco.types import Indexes
from scienco.utils import clamp

__all__ = ("automated_readability_index", "coleman_liau_index", "compute_indexes", "flesch_reading_ease_score", "Indexes")


def flesch_reading_ease_score(sentences: int, words: int, syllables: int, *, is_russian: bool = False) -> float:
    """
    Calculate the Flesch-Kincaid score.
    """
    if not sentences or not words:
        return 0.0

    # Flesch-Kincaid score constants:
    x_grade: float = FRES_RU.X_GRADE if is_russian else FRES_EN.X_GRADE
    y_grade: float = FRES_RU.Y_GRADE if is_russian else FRES_EN.Y_GRADE
    z_grade: float = FRES_RU.Z_GRADE if is_russian else FRES_EN.Z_GRADE

    value = x_grade - y_grade * (words / sentences) - z_grade * (syllables / words)
    return clamp(value, 0.0, 100.0)


def automated_readability_index(sentences: int, words: int, letters: int, *, is_russian: bool = False) -> float:
    """
    Calculate the Automated readability index.
    """
    if not sentences or not words:
        return 0.0

    # Automated readability index constants:
    x_grade: float = ARI_RU.X_GRADE if is_russian else ARI_EN.X_GRADE
    y_grade: float = ARI_RU.Y_GRADE if is_russian else ARI_EN.Y_GRADE
    z_grade: float = ARI_RU.Z_GRADE if is_russian else ARI_EN.Z_GRADE

    value = x_grade * (letters / words) + y_grade * (words / sentences) - z_grade
    return max(0.0, value)


def coleman_liau_index(sentences: int, words: int, letters: int, *, is_russian: bool = False) -> float:
    """
    Calculate the Coleman-Liau index.
    """
    if not words or words < 100:
        return 0.0

    # Coleman-Liau index constants:
    x_grade: float = CLI_RU.X_GRADE if is_russian else CLI_EN.X_GRADE
    y_grade: float = CLI_RU.Y_GRADE if is_russian else CLI_EN.Y_GRADE
    z_grade: float = CLI_RU.Z_GRADE if is_russian else CLI_EN.Z_GRADE

    value = x_grade * (letters / words * 100.0) - y_grade * (sentences / words * 100.0) - z_grade
    return max(0.0, value)


def compute_indexes(sentences: int, words: int, letters: int, syllables: int, *, is_russian: bool = False) -> Indexes:
    """
    Calculate the readability indexes.
    """
    flesch_reading_ease_score_value = flesch_reading_ease_score(sentences, words, syllables, is_russian=is_russian)
    automated_readability_index_value = automated_readability_index(sentences, words, letters, is_russian=is_russian)
    coleman_liau_index_value = coleman_liau_index(sentences, words, letters, is_russian=is_russian)
    return Indexes(
        flesch_reading_ease_score=flesch_reading_ease_score_value,
        automated_readability_index=automated_readability_index_value,
        coleman_liau_index=coleman_liau_index_value)



# -*- coding: utf-8 -*-

from itertools import chain
from itertools import product
import re
from string import ascii_letters
from typing import Iterator

from scienco.metrics.constants import cyrillic_letters
from scienco.metrics.constants import sentences_pattern as _sentences_pattern
from scienco.metrics.constants import words_pattern as _words_pattern
from scienco.types import Metrics

__all__ = ("compute_metrics", "sentences", "syllables", "words", "Metrics")

sentences_pattern = re.compile(_sentences_pattern, re.UNICODE)
words_pattern = re.compile(_words_pattern, re.UNICODE)


def sentences(string: str) -> Iterator[str]:
    """
    Tokenize a paragraph into sentences.
    """
    previous = 0

    for match in sentences_pattern.finditer(string):
        delimiter = match.group(1)
        start = match.start()

        yield string[previous:start] + delimiter
        previous = match.end()

    if previous < len(string):
        yield string[previous:]


def words(string: str) -> Iterator[str]:
    """
    Tokenize a sentence into words.
    """
    for match in words_pattern.finditer(string):
        word = match.group(1)

        if word.isalnum():
            yield word


def syllables(string: str) -> int:
    """
    Return the number of syllables in a word.
    """
    string = string.lower() if not string.islower() else string

    # Russian vowels:
    vowels = "\u0430\u0435\u0451\u0438\u043E\u0443\u044B\u044D\u044E\u044F"

    if any(vowel in string for vowel in vowels):
        return sum(map(string.count, vowels))

    # English vowels:
    vowels = "\x61\x65\x69\x6F\x75\x79"
    count = 0

    if any(vowel in string for vowel in vowels):
        count = sum(map(string.count, vowels))
        count = count - string.endswith("\x65")

        diphthongs: Iterator[str] = map("".join, product(vowels, repeat=2))
        count = count - sum(map(string.count, diphthongs))

        triphthongs: Iterator[str] = map("".join, product(vowels, repeat=3))
        count = count - sum(map(string.count, triphthongs))

        if string.endswith("\x6C\x65") or string.endswith("\x6C\x65\x73"):
            string, _ = string.split("\x6C\x65", 1)
            count = count + all(not string.endswith(vowel) for vowel in vowels)

    return max(1, count)


def compute_metrics(string: str) -> Metrics:
    """
    Calculate the metrics.
    """
    russian_letters = sum(map(string.count, cyrillic_letters))
    english_letters = sum(map(string.count, ascii_letters))

    is_russian = russian_letters > english_letters
    del english_letters, russian_letters

    sentences_list = list(sentences(string))
    sentences_count = len(sentences_list)

    words_list = list(chain.from_iterable(map(words, sentences_list)))
    words_count = len(words_list)
    del sentences_list

    letters_count = sum(map(len, words_list))
    syllables_count = sum(map(syllables, words_list))
    del words_list

    return Metrics(
        sentences=sentences_count,
        words=words_count,
        letters=letters_count,
        syllables=syllables_count,
        is_russian=is_russian)



# -*- coding: utf-8 -*-

from enum import Enum
from typing import NamedTuple

__all__ = ("FloatEnum", "Indexes", "Metrics")


class FloatEnum(float, Enum):
    """
    Enum where members are also (and must be) floats.
    """


class Metrics(NamedTuple):
    sentences: int
    words: int
    letters: int
    syllables: int
    is_russian: bool


class Indexes(NamedTuple):
    flesch_reading_ease_score: float
    automated_readability_index: float
    coleman_liau_index: float



# -*- coding: utf-8 -*-

from scienco.types import FloatEnum

__all__ = ("ARI_EN", "ARI_RU", "CLI_EN", "CLI_RU", "FRES_EN", "FRES_RU")


class FRES_EN(FloatEnum):
    """
    Flesch-Kincaid score constants for english language.
    """
    X_GRADE = 206.835
    Y_GRADE = 1.015
    Z_GRADE = 84.6


class FRES_RU(FloatEnum):
    """
    Flesch-Kincaid score constants for russian language.
    """
    X_GRADE = 220.755
    Y_GRADE = 1.315
    Z_GRADE = 50.1


class ARI_EN(FloatEnum):
    """
    Automated readability index constants for english language.
    """
    X_GRADE = 4.71
    Y_GRADE = 0.5
    Z_GRADE = 21.43


class ARI_RU(FloatEnum):
    """
    Automated readability index constants for russian language.
    """
    X_GRADE = 6.26
    Y_GRADE = 0.2805
    Z_GRADE = 31.04


class CLI_EN(FloatEnum):
    """
    Coleman-Liau index constants for english language.
    """
    X_GRADE = 0.0588
    Y_GRADE = 0.296
    Z_GRADE = 15.8


class CLI_RU(FloatEnum):
    """
    Coleman-Liau index constants for russian language.
    """
    X_GRADE = 0.055
    Y_GRADE = 0.35
    Z_GRADE = 20.33



# -*- coding: utf-8 -*-

from typing import TypeVar

__all__ = ("clamp",)

_C = TypeVar("_C", float, str, bytes)


def clamp(value: _C, min_value: _C, max_value: _C) -> _C:
    """
    Limits a provided value between two specified bounds.
    """
    return max(min_value, min(value, max_value))



# -*- coding: utf-8 -*-

"""
Calculate the readability of text using one of a variety of computed indexes.
"""

from scienco.indexes import automated_readability_index
from scienco.indexes import coleman_liau_index
from scienco.indexes import compute_indexes
from scienco.indexes import flesch_reading_ease_score
from scienco.metrics import compute_metrics
from scienco.metrics import sentences
from scienco.metrics import syllables
from scienco.metrics import words
from scienco.types import Indexes
from scienco.types import Metrics

__all__ = (
    "automated_readability_index", "coleman_liau_index", "compute_indexes", "flesch_reading_ease_score",
    "compute_metrics", "sentences", "syllables", "words",
    "Indexes", "Metrics"
)



# -*- coding: utf-8 -*-

from itertools import chain
from typing import Iterator

__all__ = ("cyrillic_letters", "cyrillic_lowercase", "cyrillic_uppercase", "sentences_pattern", "words_pattern")

sentences_pattern = "\x28\x5B\x2E\x3F\x21\u2026\x5D\x29\x5C\x73\x2B"
words_pattern = "\x28\x5B\x5E\x5C\x57\x5C\x64\x5D\x2B\x7C\x5C\x64\x2B\x7C\x5B\x5E\x5C\x77\x5C\x73\x5D\x29"

cyrillic_lowercase = "".join(map(chr, chain(range(1072, 1078), [1105], range(1078, 1104))))
cyrillic_uppercase = "".join(map(chr, chain(range(1040, 1046), [1025], range(1046, 1072))))
cyrillic_letters = cyrillic_lowercase + cyrillic_uppercase



# -*- coding: utf-8 -*-

from scienco.metrics.processing import compute_metrics
from scienco.metrics.processing import sentences
from scienco.metrics.processing import syllables
from scienco.metrics.processing import words
from scienco.types import Metrics

__all__ = ("compute_metrics", "sentences", "syllables", "words", "Metrics")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['scienco', 'scienco.indexes', 'scienco.metrics']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'scienco',
    'version': '0.2.1.1',
    'description': 'Calculate the readability of text using one of a variety of computed indexes',
    'long_description': 'Scienco\n=======\n[![pipeline status][pipeline]][homepage]\n[![coverage report][coverage]][homepage]\n[![latest version][version]][pypi]\n[![python requires][pyversions]][pypi]\n\nCalculate the readability of text using one of a variety of computed indexes including:\n\n- Flesch-Kincaid score\n- Automated readability index\n- Coleman-Liau index\n\nRequirements\n------------\nPython 3.6+\n\nInstallation\n------------\n```\n$ pip install scienco\n```\n\nUsage\n-----\n```pycon\n>>> import scienco\n>>> metrics = scienco.compute_metrics("Lorem ipsum dolor sit amet ...")\n>>> metrics\nMetrics(sentences=32, words=250, letters=1329, syllables=489, is_russian=False)\n>>> indexes = scienco.compute_indexes(sentences=32, words=250, letters=1329, syllables=489, is_russian=False)\n>>> indexes\nIndexes(flesch_reading_ease_score=33.43, automated_readability_index=7.51, coleman_liau_index=11.67)\n```\n\nDistribution\n------------\nThis project is licensed under the terms of the [MIT License](LICENSE).\n\nLinks\n-----\n- Documentation: <https://scienco.readable.pw>\n- Code: <https://gitlab.com/amalchuk/scienco>\n- GitHub mirror: <https://github.com/amalchuk/scienco>\n\n[homepage]: <https://gitlab.com/amalchuk/scienco>\n[pypi]: <https://pypi.org/project/scienco>\n[pipeline]: <https://gitlab.com/amalchuk/scienco/badges/master/pipeline.svg?style=flat-square>\n[coverage]: <https://gitlab.com/amalchuk/scienco/badges/master/coverage.svg?style=flat-square>\n[version]: <https://img.shields.io/pypi/v/scienco?color=blue&style=flat-square>\n[pyversions]: <https://img.shields.io/pypi/pyversions/scienco?color=blue&style=flat-square>\n',
    'author': 'Andrew Malchuk',
    'author_email': 'andrew.malchuk@yandex.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/amalchuk/scienco',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-

from scienco.indexes.processing import automated_readability_index
from scienco.indexes.processing import coleman_liau_index
from scienco.indexes.processing import compute_indexes
from scienco.indexes.processing import flesch_reading_ease_score
from scienco.types import Indexes

__all__ = ("automated_readability_index", "coleman_liau_index", "compute_indexes", "flesch_reading_ease_score", "Indexes")
